@@ -1,18 +1,21 @@
 package peco
 
 import (
+	"bytes"
 	"fmt"
-	"io/ioutil"
 	"os"
-	"os/exec"
+	"strconv"
 	"unicode"
 
+	"context"
+
 	"github.com/google/btree"
 	"github.com/lestrrat/go-pdebug"
 	"github.com/nsf/termbox-go"
 	"github.com/peco/peco/internal/keyseq"
+	"github.com/peco/peco/internal/util"
+	"github.com/peco/peco/line"
 	"github.com/pkg/errors"
-	"golang.org/x/net/context"
 )
 
 // This is the global map of canonical action name to actions
@@ -297,7 +300,67 @@ func doFinish(ctx context.Context, state *Peco, _ termbox.Event) {
 		defer g.End()
 	}
 
-	state.Exit(errCollectResults{})
+	ccarg := state.execOnFinish
+	if len(ccarg) == 0 {
+		state.Exit(errCollectResults{})
+		return
+	}
+
+	sel := NewSelection()
+	state.Selection().Copy(sel)
+	if sel.Len() == 0 {
+		if l, err := state.CurrentLineBuffer().LineAt(state.Location().LineNumber()); err == nil {
+			sel.Add(l)
+		}
+	}
+
+	var stdin bytes.Buffer
+	sel.Ascend(func(it btree.Item) bool {
+		line := it.(line.Line)
+		stdin.WriteString(line.Buffer())
+		stdin.WriteRune('\n')
+		return true
+	})
+
+	var err error
+	state.Hub().SendStatusMsg("Executing " + ccarg)
+	cmd := util.Shell(ccarg)
+	cmd.Stdin = &stdin
+	cmd.Stdout = state.Stdout
+	cmd.Stderr = state.Stderr
+	// Setup some enviroment variables. Start with a copy of the current
+	// environment...
+	env := os.Environ()
+
+	// Add some PECO specific ones...
+	// PECO_QUERY: current query value
+	// PECO_FILENAME: input file name, if any. "-" for stdin
+	// PECO_LINE_COUNT: number of lines in the original input
+	// PECO_MATCHED_LINE_COUNT: number of lines matched (number of lines being
+	//     sent to stdin of the command being executed)
+
+	if s, ok := state.Source().(*Source); ok {
+		env = append(env,
+			`PECO_FILENAME=`+s.Name(),
+			`PECO_LINE_COUNT=`+strconv.Itoa(s.Size()),
+		)
+	}
+
+	env = append(env,
+		`PECO_QUERY=`+state.Query().String(),
+		`PECO_MATCHED_LINE_COUNT=`+strconv.Itoa(sel.Len()),
+	)
+	cmd.Env = env
+
+	state.screen.Suspend()
+
+	err = cmd.Run()
+	state.screen.Resume()
+	state.Hub().SendDraw(&DrawOptions{DisableCache: true})
+	if err != nil {
+		// bail out, or otherwise the user cannot know what happened
+		state.Exit(errors.Wrap(err, `failed to execute command`))
+	}
 }
 
 func doCancel(ctx context.Context, state *Peco, e termbox.Event) {
@@ -314,7 +377,11 @@ func doCancel(ctx context.Context, state *Peco, e termbox.Event) {
 	}
 
 	// peco.Cancel -> end program, exit with failure
-	state.Exit(makeIgnorable(errors.New("user canceled")))
+	err := makeIgnorable(errors.New("user canceled"))
+	if state.onCancel == errorKey {
+		err = setExitStatus(err, 1)
+	}
+	state.Exit(err)
 }
 
 func doSelectDown(ctx context.Context, state *Peco, e termbox.Event) {
@@ -723,64 +790,3 @@ func makeCombinedAction(actions ...Action) ActionFunc {
 		}, toplevel)
 	})
 }
-
-func makeCommandAction(state *Peco, cc *CommandConfig) ActionFunc {
-	return func(ctx context.Context, state *Peco, _ termbox.Event) {
-		sel := state.Selection()
-		if sel.Len() == 0 {
-			if l, err := state.CurrentLineBuffer().LineAt(state.Location().LineNumber()); err == nil {
-				sel.Add(l)
-			}
-		}
-
-		sel.Ascend(func(it btree.Item) bool {
-			line := it.(Line)
-
-			var f *os.File
-			var err error
-
-			args := append([]string{}, cc.Args...)
-			for i, v := range args {
-				switch v {
-				case "$FILE":
-					if f == nil {
-						f, err = ioutil.TempFile("", "peco")
-						if err != nil {
-							return false
-						}
-						f.WriteString(line.Buffer())
-						f.Close()
-					}
-					args[i] = f.Name()
-				case "$LINE":
-					args[i] = line.Buffer()
-				}
-			}
-			state.Hub().SendStatusMsg("Executing " + cc.Name)
-			cmd := exec.Command(args[0], args[1:]...)
-			if cc.Spawn {
-				err = cmd.Start()
-				go func() {
-					cmd.Wait()
-					if f != nil {
-						os.Remove(f.Name())
-					}
-				}()
-			} else {
-				cmd.Stdin = state.Stdin
-				cmd.Stdout = state.Stdout
-				cmd.Stderr = state.Stderr
-				err = cmd.Run()
-				if f != nil {
-					os.Remove(f.Name())
-				}
-				state.ExecQuery()
-			}
-			if err != nil {
-				return false
-			}
-
-			return true
-		})
-	}
-}
