@@ -1,14 +1,15 @@
 package main
 
 import (
-	"bytes"
 	"fmt"
 	"os"
 	"runtime"
 
+	"context"
+
+	pdebug "github.com/lestrrat/go-pdebug"
 	"github.com/peco/peco"
 	"github.com/peco/peco/internal/util"
-	"golang.org/x/net/context"
 )
 
 func main() {
@@ -22,37 +23,32 @@ func main() {
 }
 
 func _main() int {
+	if pdebug.Enabled {
+		pdebug.DefaultCtx.Writer = os.Stderr
+	}
 	if envvar := os.Getenv("GOMAXPROCS"); envvar == "" {
 		runtime.GOMAXPROCS(runtime.NumCPU())
 	}
-	ctx := context.Background()
+	ctx, cancel := context.WithCancel(context.Background())
+	defer cancel()
 
 	cli := peco.New()
 	if err := cli.Run(ctx); err != nil {
 		switch {
 		case util.IsCollectResultsError(err):
-			selection := cli.Selection()
-			if selection.Len() == 0 {
-				if l, err := cli.CurrentLineBuffer().LineAt(cli.Location().LineNumber()); err == nil {
-					selection.Add(l)
-				}
-			}
-
-			cli.SetResultCh(make(chan peco.Line))
-			go cli.CollectResults()
+			cli.PrintResults()
+			return 0
 		case util.IsIgnorableError(err):
+			if st, ok := util.GetExitStatus(err); ok {
+				return st
+			}
 			return 0
 		default:
 			fmt.Fprintf(os.Stderr, "Error: %s\n", err)
 			return 1
 		}
+
 	}
 
-	buf := bytes.Buffer{}
-	for line := range cli.ResultCh() {
-		buf.WriteString(line.Output())
-		buf.WriteByte('\n')
-	}
-	os.Stdout.Write(buf.Bytes())
 	return 0
 }
