@@ -3,12 +3,14 @@ package peco
 import (
 	"fmt"
 	"strconv"
+	"strings"
 	"time"
 	"unicode/utf8"
 
 	"github.com/lestrrat/go-pdebug"
 	"github.com/mattn/go-runewidth"
 	"github.com/nsf/termbox-go"
+	"github.com/peco/peco/line"
 	"github.com/pkg/errors"
 )
 
@@ -280,7 +282,7 @@ func (s *StatusBar) PrintStatus(msg string, clearDelay time.Duration) {
 func NewListArea(screen Screen, anchor VerticalAnchor, anchorOffset int, sortTopDown bool, styles *StyleSet) *ListArea {
 	return &ListArea{
 		AnchorSettings: NewAnchorSettings(screen, anchor, anchorOffset),
-		displayCache:   []Line{},
+		displayCache:   []line.Line{},
 		dirty:          false,
 		sortTopDown:    sortTopDown,
 		styles:         styles,
@@ -288,7 +290,7 @@ func NewListArea(screen Screen, anchor VerticalAnchor, anchorOffset int, sortTop
 }
 
 func (l *ListArea) purgeDisplayCache() {
-	l.displayCache = []Line{}
+	l.displayCache = []line.Line{}
 }
 
 func (l *ListArea) IsDirty() bool {
@@ -310,6 +312,7 @@ type DrawOptions struct {
 	RunningQuery bool
 	DisableCache bool
 }
+
 // Draw displays the ListArea on the screen
 func (l *ListArea) Draw(state *Peco, parent Layout, perPage int, options *DrawOptions) {
 	if pdebug.Enabled {
@@ -361,10 +364,18 @@ func (l *ListArea) Draw(state *Peco, parent Layout, perPage int, options *DrawOp
 		loc.SetLineNumber(lbufsiz - 1)
 	}
 
+	// The max column size is calculated by buf. we check against where the
+	// loc variable thinks we should be scrolling to, and make sure that this
+	// falls in range with what we got
+	width, _ := state.screen.Size()
+	if max := maxOf(buf.MaxColumn()-width, 0); loc.Column() > max {
+		loc.SetColumn(max)
+	}
+
 	// previously drawn lines are cached. first, truncate the cache
 	// to current size of the drawable area
 	if ldc := int(len(l.displayCache)); ldc != perPage {
-		newCache := make([]Line, perPage)
+		newCache := make([]line.Line, perPage)
 		copy(newCache, l.displayCache)
 		l.displayCache = newCache
 	} else if perPage > bufsiz {
@@ -397,17 +408,40 @@ func (l *ListArea) Draw(state *Peco, parent Layout, perPage int, options *DrawOp
 
 	var cached, written int
 	var fgAttr, bgAttr termbox.Attribute
+	var selectionPrefix = state.selectionPrefix
+	var prefix = ""
+
+	var prefixCurrentSelection string
+	var prefixSavedSelection string
+	var prefixDefault string
+	if len := len(selectionPrefix); len > 0 {
+		prefixCurrentSelection = selectionPrefix + " "
+		prefixSavedSelection = "*" + strings.Repeat(" ", len)
+		prefixDefault = strings.Repeat(" ", len+1)
+	}
+
 	for n := 0; n < perPage; n++ {
-		switch {
-		case n+loc.Offset() == loc.LineNumber():
-			fgAttr = l.styles.Selected.fg
-			bgAttr = l.styles.Selected.bg
-		case selectionContains(state, n+loc.Offset()):
-			fgAttr = l.styles.SavedSelection.fg
-			bgAttr = l.styles.SavedSelection.bg
-		default:
-			fgAttr = l.styles.Basic.fg
-			bgAttr = l.styles.Basic.bg
+		if len(selectionPrefix) > 0 {
+			switch {
+			case n+loc.Offset() == loc.LineNumber():
+				prefix = prefixCurrentSelection
+			case selectionContains(state, n+loc.Offset()):
+				prefix = prefixSavedSelection
+			default:
+				prefix = prefixDefault
+			}
+		} else {
+			switch {
+			case n+loc.Offset() == loc.LineNumber():
+				fgAttr = l.styles.Selected.fg
+				bgAttr = l.styles.Selected.bg
+			case selectionContains(state, n+loc.Offset()):
+				fgAttr = l.styles.SavedSelection.fg
+				bgAttr = l.styles.SavedSelection.bg
+			default:
+				fgAttr = l.styles.Basic.fg
+				bgAttr = l.styles.Basic.bg
+			}
 		}
 
 		if n >= bufsiz {
@@ -439,8 +473,17 @@ func (l *ListArea) Draw(state *Peco, parent Layout, perPage int, options *DrawOp
 		xOffset := loc.Column()
 		line := target.DisplayString()
 
-		pdebug.Printf("state.SingleKeyJumpMode = %t", state.SingleKeyJumpMode())
-		pdebug.Printf("state.SingleKeyJumpShowPrefix = %t", state.SingleKeyJumpShowPrefix())
+		if len := len(prefix); len > 0 {
+			l.screen.Print(PrintArgs{
+				X:       x,
+				Y:       y,
+				XOffset: xOffset,
+				Fg:      fgAttr,
+				Bg:      bgAttr,
+				Msg:     prefix,
+			})
+			x += len
+		}
 		if state.SingleKeyJumpMode() || state.SingleKeyJumpShowPrefix() {
 			prefixes := state.SingleKeyJumpPrefixes()
 			if n < int(len(prefixes)) {
@@ -474,8 +517,8 @@ func (l *ListArea) Draw(state *Peco, parent Layout, perPage int, options *DrawOp
 			x += 2
 		}
 
-		matches := target.Indices()
-		if matches == nil {
+		ix, ok := target.(MatchIndexer)
+		if !ok {
 			l.screen.Print(PrintArgs{
 				X:       x,
 				Y:       y,
@@ -488,6 +531,7 @@ func (l *ListArea) Draw(state *Peco, parent Layout, perPage int, options *DrawOp
 			continue
 		}
 
+		matches := ix.Indices()
 		prev := x
 		index := 0
 
@@ -549,6 +593,13 @@ func (l *ListArea) Draw(state *Peco, parent Layout, perPage int, options *DrawOp
 	}
 }
 
+func maxOf(a, b int) int {
+	if a > b {
+		return a
+	}
+	return b
+}
+
 // NewDefaultLayout creates a new Layout in the default format (top-down)
 func NewDefaultLayout(state *Peco) *BasicLayout {
 	return &BasicLayout{
@@ -647,8 +698,8 @@ func (l *BasicLayout) linesPerPage() int {
 		if pdebug.Enabled {
 			pdebug.Printf(
 				"linesPerPage is < 1 (height = %d, reservedLines = %d), forcing return value of 2",
-				strconv.Itoa(height),
-				strconv.Itoa(reservedLines),
+				height,
+				reservedLines,
 			)
 		}
 		return 2
